import {
  ClassType,
  PropertyType,
} from '@loafer/core/constants/types';

import {
  Assert,
} from '@loafer/core/util';

import {
  Annotation,
} from '@loafer/core/annotation';

import {
  PouchDefinitionRegistry,
} from '@loafer/pouches/factory/support';


import GenericApplicationContext from '@loafer/context/support/GenericApplicationContext';

export class AnnotatedPouchDefinitionReader {
	private readonly registry: PouchDefinitionRegistry;
	private pouchNameGenerator: PouchNameGenerator = new AnnotationPouchNameGenerator();
	private scopeMetadataResolver: ScopeMetadataResolver = new AnnotationScopeMetadataResolver();
	private conditionEvaluator: ConditionEvaluator;


	public constructor(registry: PouchDefinitionRegistry, environment?: Environment) {
    if (undefined === environment) {
      environment = this.getOrCreateEnvironment(registry);
    }
    Assert.notNull(registry, 'PouchDefinitionRegistry must not be null');
    Assert.notNull(environment, 'Environment must not be null');
    this.registry = registry;
    this.conditionEvaluator = new ConditionEvaluator(registry, environment, null);
    AnnotationConfigUtils.registerAnnotationConfigProcessors(this.registry);
	}

	public getRegistry(): PouchDefinitionRegistry {
		return this.registry;
	}

	public setEnvironment(environment: Environment): void {
		this.conditionEvaluator = new ConditionEvaluator(this.registry, environment, null);
	}

	public setPouchNameGenerator(pouchNameGenerator: PouchNameGenerator): void {
		this.pouchNameGenerator = (pouchNameGenerator !== null ? pouchNameGenerator : new AnnotationPouchNameGenerator());
	}

	public setScopeMetadataResolver(scopeMetadataResolver: ScopeMetadataResolver): void {
		this.scopeMetadataResolver =
				(scopeMetadataResolver !== null ? scopeMetadataResolver : new AnnotationScopeMetadataResolver());
	}


	public register(...annotatedClasses: ClassType[]): void {
    annotatedClasses.forEach(annotatedClass => {
      this.registerPouch(annotatedClass);
    });
	}

	public registerPouch<TAnnotation extends Annotation>(annotatedClass: ClassType, name: PropertyType, ...qualifiers: TAnnotation[]): void {
		let abd: AnnotatedGenericPouchDefinition = new AnnotatedGenericPouchDefinition(annotatedClass);
		if (this.conditionEvaluator.shouldSkip(abd.getMetadata())) {
			return;
		}

		let scopeMetadata: ScopeMetadata = this.scopeMetadataResolver.resolveScopeMetadata(abd);
		abd.setScope(scopeMetadata.getScopeName());
		let pouchName: string = (name !== null ? name : this.pouchNameGenerator.generatePouchName(abd, this.registry));
		AnnotationConfigUtils.processCommonDefinitionAnnotations(abd);
		if (qualifiers !== undefined) {
      qualifiers.forEach((qualifier, index, array) => {
				if (Primary.class === qualifier) {
					abd.setPrimary(true);
				} else if (Lazy.class === qualifier) {
					abd.setLazyInit(true);
				} else {
					abd.addQualifier(new AutowireCandidateQualifier(qualifier));
				}
      });
		}

		let definitionHolder: PouchDefinitionHolder = new PouchDefinitionHolder(abd, pouchName);
		definitionHolder = AnnotationConfigUtils.applyScopedProxyMode(scopeMetadata, definitionHolder, this.registry);
		PouchDefinitionReaderUtils.registerPouchDefinition(definitionHolder, this.registry);
	}


	private static getOrCreateEnvironment(registry: PouchDefinitionRegistry): Environment {
		Assert.notNull(registry, 'PouchDefinitionRegistry must not be null');
		if (registry instanceof EnvironmentCapable) {
			return (<EnvironmentCapable>registry).getEnvironment();
		}
		return new StandardEnvironment();
	}
}

export default AnnotatedPouchDefinitionReader;
