import {
  ClassType,
  Construtorable,
} from '@loafer/core/constants/types';
import * as ReflectConstants from '@loafer/core/constants/reflect';
import Reflection from '@loafer/core/reflect/Reflection';
import Class from '@loafer/core/reflect/Class';
import Annotation from '@loafer/core/annotation/Annotation';
import DecoratorType from './DecoratorType';

export class Decorator {
  public static create = (AnnotationType: Construtorable<Annotation>) => {
    return (...handlerArgs: any[]) => {
      let annotation: Annotation = new AnnotationType(...handlerArgs);

      return (...decoratorArgs: any[]) => {
        let decoratorType: DecoratorType = Decorator._detectDecoratorType(name, annotation, decoratorArgs);
        let type = typeof decoratorArgs[0] === 'function' ? decoratorArgs[0].prototype : decoratorArgs[0];

        let reflection: Reflection = Reflect.getMetadata(ReflectConstants.REFLECT_META, type);
        if (undefined === reflection) {
          reflection = new Reflection(type);
          Reflect.defineMetadata(ReflectConstants.REFLECT_META, reflection, type);
        }
        let clazz: Class = reflection.getClass();

        switch(decoratorType) {
        case DecoratorType.CLASS:
          reflection.addClassAnnotation(annotation);
          return annotation.onClassDecorator.call(annotation, clazz, decoratorArgs[0]);
        case DecoratorType.PROPERTY:
          reflection.addPropertyAnnotation(annotation, decoratorArgs[1]);
          return annotation.onPropertyDecorator.call(annotation, clazz, decoratorArgs[0], decoratorArgs[1]);
        case DecoratorType.METHOD:
          reflection.addMethodAnnotation(annotation, decoratorArgs[1]);
          return annotation.onMethodDecorator.call(annotation, clazz, decoratorArgs[0], decoratorArgs[1], decoratorArgs[2]);
        case DecoratorType.PARAMETER:
          reflection.addParameterAnnotation(annotation, decoratorArgs[1], decoratorArgs[2]);
          return annotation.onParameterDecorator.call(annotation, clazz, decoratorArgs[0], decoratorArgs[1], decoratorArgs[2]);
        default:
        }
      };
    };
  }

  public static get = (targetType: ClassType) => {
    let reflection: Reflection = Reflect.getMetadata(ReflectConstants.REFLECT_META, targetType);
    if (undefined === reflection) {
      return undefined;
    }
    return reflection.getClass();
  }

  private static _detectDecoratorType(name: string, annotation: any, args: any[]): DecoratorType {
    let params = [];
    for (let i = 0; i < args.length; i++) {
      if (args[i]) {
        params.push(args[i]);
      }
    }
    const paramCount = params.length;
    let decoratorType: DecoratorType;

    if (1 === paramCount) {
      if ('onClassDecorator' in annotation) {
        throw new Error(`Cannot apply @${name} decorator on Class.`);
      }
      decoratorType = DecoratorType.CLASS;
    } else if (2 === paramCount) {
      if ('onPropertyDecorator' in annotation) {
        throw new Error(`Cannot apply @${name} decorator on Property.`);
      }
      decoratorType = DecoratorType.PROPERTY;
    } else if (3 === paramCount) {
      if(typeof args[2] === 'number') {
        if ('onParameterDecorator' in annotation) {
          throw new Error(`Cannot apply @${name} decorator on Parameter.`);
        }
        decoratorType = DecoratorType.PARAMETER;
      } else {
        if ('onMethodDecorator' in annotation) {
          throw new Error(`Cannot apply @${name} decorator on Method.`);
        }
        decoratorType = DecoratorType.METHOD;
      }
    } else {
      throw new Error(`@${name} decorator is not valid here!`);
    }

    return decoratorType;
  }
}
