import {
  ClassType,
} from '@loafer/core/constants/types';

import {
  IllegalStateException,
  IllegalArgumentException,
} from '@loafer/core/lang';

import {
  StringUtils,
} from '@loafer/core/util';

export abstract class Assert {
	public static state(expression: boolean, message: string): void {
		if (!expression) {
			throw new IllegalStateException(message);
		}
	}

  public static isTrue(expression: boolean, message: string): void {
		if (!expression) {
			throw new IllegalArgumentException(message);
		}
	}

	public static isNull(object: any, message: string): void {
		if (object !== null) {
			throw new IllegalArgumentException(message);
		}
	}

	public static notNull(object: any, message: string): void {
		if (object == null) {
			throw new IllegalArgumentException(message);
		}
	}

	public static hasLength(text: string, message: string): void {
		if (!StringUtils.hasLength(text)) {
			throw new IllegalArgumentException(message);
		}
	}

	public static hasText(text: string, message: string): void {
		if (!StringUtils.hasText(text)) {
			throw new IllegalArgumentException(message);
		}
	}


	public static doesNotContain(textToSearch: string, substring: string, message: string): void {
		if (StringUtils.hasLength(textToSearch) && StringUtils.hasLength(substring) &&
				textToSearch.includes(substring)) {
			throw new IllegalArgumentException(message);
		}
	}

	public static notEmpty(collection: any[] | Map<any, any> | Set<any>, message: string): void {
    if (undefined === collection || null === collection) {
      throw new IllegalArgumentException(message);
    }
    let valid = true;
    if (collection instanceof Array) {
      let array: any[] = collection;
      if (0 === collection.length) {
        valid = false;
      }
    } else if (collection instanceof Map) {
      let map: Map<any, any> = collection;
      if (0 === map.size) {
        valid = false;
      }
    } else if (collection instanceof Set) {
      let set: Set<any> = collection;
      if (0 === set.size) {
        valid = false;
      }
    } else {
      valid = false;
    }
    if (!valid) {
      throw new IllegalArgumentException(message);
    }
	}

	public static noNullElements(array: any[], message: string): void {
		if (array !== null) {
      array.forEach(element => {
				if (element === null) {
					throw new IllegalArgumentException(message);
				}
      });
		}
	}

	public static isInstanceOf(type: ClassType, obj: any, message: string = ''): void {
		Assert.notNull(type, 'Type to check against must not be null');
		if (!(obj instanceof type.constructor)) {
			Assert.instanceCheckFailed(type, obj, message);
		}
	}

	private static instanceCheckFailed(type: ClassType, obj: any, message: string): void {
		let className = (obj !== undefined && obj !== null ? Object.getPrototypeOf(obj).constructor.name : 'null');
		let result = '';
		let defaultMessage = true;
		if (StringUtils.hasLength(message)) {
			if (Assert.endsWithSeparator(message)) {
				result = message + ' ';
			} else {
				result = Assert.messageWithTypeName(message, className);
				defaultMessage = false;
			}
		}
		if (defaultMessage) {
			result = result + (`Object of class [${className}] must be an instance of ${type}`);
		}
		throw new IllegalArgumentException(result);
	}

	private static endsWithSeparator(message: string): boolean {
		return (message.endsWith(':') || message.endsWith(';') || message.endsWith(',') || message.endsWith('.'));
	}

	private static messageWithTypeName(message: string, typeName: string): string {
		return message + (message.endsWith(' ') ? '' : ': ') + typeName;
	}

}

export default Assert;
