import {
  IllegalStateException,
  IllegalArgumentException,
} from '@loafer/core/lang';


export abstract class StringUtils {
	private static readonly FOLDER_SEPARATOR: string = '/';

	private static readonly WINDOWS_FOLDER_SEPARATOR: string = '\\';

	private static readonly TOP_PATH: string = '..';

	private static readonly CURRENT_PATH: string = '.';

	private static readonly EXTENSION_SEPARATOR: string = '.';


	public static isEmpty(str: any): boolean {
		return (undefined === str || null === str || '' === str);
	}

	public static hasLength(str: string): boolean {
		return (undefined !== str && null !== str && str.length > 0);
	}

	public static hasText(str: string): boolean {
    if (!StringUtils.hasLength(str)) {
      return false;
    }
    return str.trim() !== '';
	}

	public static containsWhitespace(str: string): boolean {
		if (!StringUtils.hasLength(str)) {
			return false;
		}

		return str.replace(/^\s+|\s+$/gm, '').length === 0;
	}

	public static trimWhitespace(str: string): string {
		if (!StringUtils.hasLength(str)) {
			return str;
		}

		return str.trim();
	}

	public static trimAllWhitespace(str: string): string {
		if (!StringUtils.hasLength(str)) {
			return str;
		}

		return str.replace(/\s/g, '');
	}

	public static trimLeadingWhitespace(str: string): string {
    if (!StringUtils.hasLength(str)) {
      return str;
    }

    return str.replace(/^\s+/gm, '');
  }

	public static trimTrailingWhitespace(str: string): string {
    if (!StringUtils.hasLength(str)) {
      return str;
    }

    return str.replace(/^\s+$/gm, '');
	}

	public static trimLeadingCharacter(str: string, leadingCharacter: string): string {
		if (!StringUtils.hasLength(str)) {
			return str;
		}

		return str.replace(new RegExp('^[' + leadingCharacter + ']+'), '');
	}

	public static trimTrailingCharacter(str: string, trailingCharacter: string): string {
		if (!StringUtils.hasLength(str)) {
			return str;
		}

		return str.replace(new RegExp('[' + trailingCharacter + ']+$'), '');
	}


	public static startsWithIgnoreCase(str: string, prefix: string): boolean {
		if (undefined === str || null === str || undefined === prefix || null === prefix) {
			return false;
		}
		if (str.startsWith(prefix)) {
			return true;
		}
		if (str.length < prefix.length) {
			return false;
		}

		const lcStr = str.substring(0, prefix.length).toLowerCase();
		const lcPrefix = prefix.toLowerCase();
		return lcStr === lcPrefix;
	}

	public static endsWithIgnoreCase(str: string, suffix: string): boolean {
		if (str == null || suffix == null) {
			return false;
		}
		if (str.endsWith(suffix)) {
			return true;
		}
		if (str.length < suffix.length) {
			return false;
		}

		const lcStr = str.substring(str.length - suffix.length).toLowerCase();
		const lcSuffix = suffix.toLowerCase();
		return lcStr === lcSuffix;
	}

	public static substringMatch(str: string, index: number, substring: string): boolean {
    return str.includes(substring);
	}

	public static countOccurrencesOf(str: string, sub: string): number {
    if (!StringUtils.hasLength(str) || !StringUtils.hasLength(sub)) {
      return 0;
    }

    return (str.match(new RegExp(sub, 'gi')) || []).length;
	}

	public static replace(inString: string, oldPattern: string, newPattern: string): string {
		if (!StringUtils.hasLength(inString) || !StringUtils.hasLength(oldPattern) || newPattern == null) {
			return inString;
		}
		return inString.replace(new RegExp(oldPattern, 'g'), newPattern);
	}

	public static delete(inString: string, pattern: string): string {
		return StringUtils.replace(inString, pattern, '');
	}

	public static deleteAny(inString: string, charsToDelete: string): string {
		if (!StringUtils.hasLength(inString) || !StringUtils.hasLength(charsToDelete)) {
			return inString;
		}

		return inString.replace(new RegExp('^[' + charsToDelete + ']+'), '');
	}


	public static quote(str: string): string {
		return (str !== null ? '\'' + str + '\'' : null);
	}

	public static quoteIfString(obj: any): any {
		return (typeof obj === 'string' ? StringUtils.quote(obj) : obj);
	}

	public static unqualify(qualifiedName: string, separator: string = '.'): string {
		return qualifiedName.substring(qualifiedName.lastIndexOf(separator) + 1);
	}


	public static capitalize(str: string): string {
		return StringUtils.changeFirstCharacterCase(str, true);
	}

	public static uncapitalize(str: string): string {
		return StringUtils.changeFirstCharacterCase(str, false);
	}

	private static changeFirstCharacterCase(str: string, capitalize: boolean): string {
    if (!StringUtils.hasLength(str)) {
      return str;
    }

    return capitalize ? str.charAt(0).toUpperCase() + str.slice(1) : str.charAt(0).toLowerCase() + str.slice(1);
	}

	public static getFilename(path: string): string {
    if (path === null) {
      return null;
    }

    return path.split(StringUtils.WINDOWS_FOLDER_SEPARATOR).pop().split(StringUtils.FOLDER_SEPARATOR).pop();
	}

	public static getFilenameExtension(path: string): string {
		if (path == null) {
			return null;
		}

		return StringUtils.getFilename(path).split(StringUtils.EXTENSION_SEPARATOR).pop();
	}

	public static stripFilenameExtension(path: string): string {
		if (path == null) {
			return null;
		}

		return path.replace(new RegExp('[' + StringUtils.EXTENSION_SEPARATOR + StringUtils.getFilenameExtension(path) + ']+$'), '');
	}

	public static applyRelativePath(path: string, relativePath: string): string {
		let separatorIndex = path.lastIndexOf(StringUtils.FOLDER_SEPARATOR);
		if (separatorIndex !== -1) {
			let newPath = path.substring(0, separatorIndex);
			if (!relativePath.startsWith(StringUtils.FOLDER_SEPARATOR)) {
				newPath += StringUtils.FOLDER_SEPARATOR;
			}
			return newPath + relativePath;
		} else {
			return relativePath;
		}
	}

}

export default StringUtils;
