import {
  PropertyType,
} from '@loafer/core/constants/types';
import AliasRegistry from '@loafer/core/AliasRegistry';
import PouchDefinition from '@loafer/pouches/factory/config/PouchDefinition';
import {
  PouchDefinitionStoreException,
  NoSuchPouchDefinitionException,
} from '@loafer/pouches/factory';


export interface PouchDefinitionRegistry extends AliasRegistry {

	/**
	 * Register a new pouch definition with this registry.
	 * Must support RootPouchDefinition and ChildPouchDefinition.
	 * @param pouchName the name of the pouch instance to register
	 * @param pouchDefinition definition of the pouch instance to register
	 * @throws PouchDefinitionStoreException if the PouchDefinition is invalid
	 * or if there is already a PouchDefinition for the specified pouch name
	 * (and we are not allowed to override it)
	 * @see RootPouchDefinition
	 * @see ChildPouchDefinition
	 */
	registerPouchDefinition(pouchName: PropertyType, pouchDefinition: PouchDefinition): void;

	/**
	 * Remove the PouchDefinition for the given name.
	 * @param pouchName the name of the pouch instance to register
	 * @throws NoSuchPouchDefinitionException if there is no such pouch definition
	 */
	 removePouchDefinition(pouchName: PropertyType): void;

	/**
	 * Return the PouchDefinition for the given pouch name.
	 * @param pouchName name of the pouch to find a definition for
	 * @return the PouchDefinition for the given name (never {@code null})
	 * @throws NoSuchPouchDefinitionException if there is no such pouch definition
	 */
	getPouchDefinition(pouchName: PropertyType): PouchDefinition;

	/**
	 * Check if this registry contains a pouch definition with the given name.
	 * @param pouchName the name of the pouch to look for
	 * @return if this registry contains a pouch definition with the given name
	 */
	containsPouchDefinition(pouchName: PropertyType): boolean;

	/**
	 * Return the names of all pouchs defined in this registry.
	 * @return the names of all pouchs defined in this registry,
	 * or an empty array if none defined
	 */
	getPouchDefinitionNames(): PropertyType[];

	/**
	 * Return the number of pouchs defined in the registry.
	 * @return the number of pouchs defined in the registry
	 */
	getPouchDefinitionCount(): number;

	/**
	 * Determine whether the given pouch name is already in use within this registry,
	 * i.e. whether there is a local pouch or alias registered under this name.
	 * @param pouchName the name to check
	 * @return whether the given pouch name is already in use
	 */
	isPouchNameInUse(pouchName: PropertyType): boolean;
}

export default PouchDefinitionRegistry;
